/* Copy-pasted and adapted from:
http://www.gstreamer.net/data/doc/gstreamer/head/manual/html/chapter-helloworld.html#section-helloworld

All rights as reported in the page.*/

#include <gst/gst.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#define PIPELINE_FORMAT "\
gstrtpbin name=rtpbin \
filesrc location=%s blocksize=%d ! \
video/x-raw-yuv, width=%d, height=%d, framerate=(fraction)15/1, format=(fourcc)I420 ! \
x264enc ! \
rtph264pay ! \
rtpbin.send_rtp_sink_0 \
rtpbin.send_rtp_src_0 ! udpsink host=%s port=5000 \
rtpbin.send_rtcp_src_0 ! udpsink host=%s port=5001 sync=false async=false \
udpsrc port=5005 ! rtpbin.recv_rtcp_sink_0 \
"

static gboolean
bus_call (GstBus     *bus,
          GstMessage *msg,
          gpointer    data)
{
  GMainLoop *loop = (GMainLoop *) data;

  switch (GST_MESSAGE_TYPE (msg)) {

    case GST_MESSAGE_EOS:
      g_print ("End of stream\n");
      g_main_loop_quit (loop);
      break;

    case GST_MESSAGE_ERROR: {
      gchar  *debug;
      GError *error;

      gst_message_parse_error (msg, &error, &debug);
      g_free (debug);

      g_printerr ("Error: %s\n", error->message);
      g_error_free (error);

      g_main_loop_quit (loop);
      break;
    }
    default:
      break;
  }

  return TRUE;
}


int
main (int   argc,
      char *argv[])
{
  GMainLoop *loop;
  int width, height;

  GstElement *pipeline;
  GError *error = NULL;
  GstBus *bus;
  char * pipeline_desc;

  /* Initialisation */
  gst_init (&argc, &argv);

  loop = g_main_loop_new (NULL, FALSE);


  /* Check input arguments */
  if (argc != 5) {
    g_printerr ("Usage: %s <yuv 420 filename> <width> <height> <ip address>\n", argv[0]);
    return -1;
  }
  sscanf(argv[2], "%d", &width);
  sscanf(argv[3], "%d", &height);

  if (width >= 10000 || height >= 10000){
    g_printerr ("widht and height must be lower than 10000\n");
    return -1;
  }

  pipeline_desc = malloc(sizeof (PIPELINE_FORMAT) + strlen(argv[1]) + strlen(argv[4])*2 + 16);

  sprintf(
      pipeline_desc, 
      PIPELINE_FORMAT, 
      argv[1], 
      width*height*3/2, 
      width, 
      height, 
      argv[4], 
      argv[4]);

  printf("pipeline: %s\n", pipeline_desc);

  /* Create gstreamer elements */
  pipeline = gst_parse_launch (pipeline_desc, &error);

  /* TODO: Handle recoverable errors. */

  free(pipeline_desc);

  if (!pipeline) {
    g_printerr ("Pipeline could not be created. Exiting.\n");
    return -1;
  }

  /* Set up the pipeline */

  /* we add a message handler */
  bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
  gst_bus_add_watch (bus, bus_call, loop);
  gst_object_unref (bus);

  /* Set the pipeline to "playing" state*/
  g_print ("Now playing: %s\n", argv[1]);
  gst_element_set_state (pipeline, GST_STATE_PLAYING);

  /* Iterate */
  g_print ("Running...\n");
  g_main_loop_run (loop);

  /* Out of the main loop, clean up nicely */
  g_print ("Returned, stopping playback\n");
  gst_element_set_state (pipeline, GST_STATE_NULL);

  g_print ("Deleting pipeline\n");
  gst_object_unref (GST_OBJECT (pipeline));

  return 0;
}

